use std::{path::PathBuf, sync::Arc};

use clap::{Arg, ArgAction, ArgMatches, Command};
use octopush_core::{
    git::{git::LocalGitProviderOptions, gitea::client::DefaultGiteaClientOptions},
    schema,
};
use octopush_infra::service_register::ServiceRegister;
use tokio::sync::Mutex;

pub fn execute_cmd() -> Command {
    Command::new("execute")
        .about("execute a certain action")
        .arg(
            Arg::new("action")
                .long("action")
                .short('a')
                .action(ArgAction::Set)
                .help("action path to your local octopush.yaml file")
                .long_help("action path to your local octopush.yaml file")
                .required(true),
        )
        .arg(
            Arg::new("gitea-http-token")
                .long("gitea-http-token")
                .action(ArgAction::Set)
                .required(false),
        )
}

pub async fn execute_subcommand(args: &ArgMatches) -> eyre::Result<()> {
    let action = args
        .get_one::<String>("action")
        .ok_or(eyre::anyhow!("--action is required"))?;

    let gitea_http_token = args.get_one::<String>("gitea-http-token");

    let service_register = ServiceRegister::new(
        LocalGitProviderOptions {
            http_auth: gitea_http_token.map(|t| t.clone()),
        },
        DefaultGiteaClientOptions {
            url: "https://git.front.kjuulh.io/api/v1".into(),
            basicauth: Some("kjuulh:".into()),
        },
    );

    let action_path: PathBuf = action.into();

    let schema = service_register
        .schema_parser
        .parse_file(action_path.join("octopush.yml"))
        .await?;

    match schema {
        schema::models::Schema::Action {
            name,
            select,
            action,
        } => {
            tracing::debug!(name, "running action");

            tracing::info!("fetching repos");
            let mut git_paths = Vec::new();
            if let Some(git) = select.git.clone() {
                let mut repo_clones = Vec::with_capacity(git.repositories.len());
                for repo in git.repositories {
                    let gp = service_register.git_provider.clone();
                    repo_clones.push(tokio::spawn(async move { gp.clone_from_url(repo).await }));
                }

                for repo_clone in repo_clones {
                    let path = repo_clone.await??;
                    git_paths.push(path);
                }
            }

            let mut gitea_paths = Vec::new();
            if let Some(gitea) = select.gitea.clone() {
                let mut repo_clones = Vec::with_capacity(gitea.repositories.len());
                for repo in gitea.repositories {
                    let gp = service_register.gitea_provider.clone();
                    repo_clones.push(tokio::spawn(async move {
                        gp.clone_from_qualified(repo.clone())
                            .await
                            .map(|(p, r)| (p, r, repo))
                    }))
                }

                for repo_clone in repo_clones {
                    let path = repo_clone.await??;
                    gitea_paths.push(path);
                }
            }

            for (path, repo) in git_paths {
                let repo = Arc::new(Mutex::new(repo));
                if let Some(git) = select.git.clone() {
                    if let Some(push) = git.push {
                        service_register
                            .git_provider
                            .create_branch(repo.clone(), &push.branch.name)
                            .await?;
                    }
                }

                service_register
                    .executor
                    .execute(path.clone(), action_path.clone(), action.clone())
                    .await?;

                if let Some(git) = select.git.clone() {
                    if let Some(push) = git.push {
                        service_register
                            .git_provider
                            .push_branch(repo, &push.branch.name)
                            .await?;
                    }
                }
            }

            for (path, repo, repo_name) in gitea_paths {
                let repo = Arc::new(Mutex::new(repo));
                if let Some(gitea) = select.gitea.clone() {
                    if let Some(push) = gitea.push {
                        service_register
                            .gitea_provider
                            .create_branch(repo.clone(), &push.pull_request)
                            .await?;
                    }
                }

                service_register
                    .executor
                    .execute(path.clone(), action_path.clone(), action.clone())
                    .await?;

                if let Some(gitea) = select.gitea.clone() {
                    if let Some(push) = gitea.push {
                        service_register
                            .gitea_provider
                            .create_pull_request(repo, &repo_name, &push.pull_request)
                            .await?;
                    }
                }
            }
        }
    }

    service_register.cleanup().await?;

    Ok(())
}
